unit dsdmusic;

{*************TDSDMusic************
Extremely specialized DirectMusic wrapper
Made in Delphi 4

Author:  Dag Sverre Seljebotn, dagsverre@sensewave.com
Created: 25-04-2000
History: 25-04-2000 First release

Needs DirectX 7.0 headers for Delphi. You can find them on www.delphi-jedi.org,
under the Graphics section (bottom-left...)

Created for the FF8 Configurator v. 2, http://ffsf.cjb.net

This code is distributed under the GNU General Public License.
NO WARRANTY. See COPYING for details.

Copyright (c) 2000 Dag Sverre Seljebotn
}

interface

uses DirectMusic, Classes;

type
  TDSDMusic = class
  private
    FPerformance:  IDirectMusicPerformance;
    FSegment:      IDirectMusicSegment;
    FSegmentState: IDirectMusicSegmentState;

    FLength,
    FPausedPos: Integer;

    FDeviceList: TStringList;
    function GetDeviceList: TStringList;
    function GetDeviceCaps(I: Integer): TDMus_PortCaps;
    function GetIsPlaying: Boolean;
    function GetTrackPosition: Integer;
    procedure SetTrackPosition(const Value: Integer);
    function GetIsPaused: Boolean;
  public
    destructor Destroy; override;

    procedure PlayTrack(Device: Integer; SGT, DLS: String; Effects: Cardinal);
    procedure ResumeTrack;
    procedure PauseTrack;
    procedure StopTrack;

    property DeviceList: TStringList read GetDeviceList;
    property DeviceCaps[I: Integer]: TDMus_PortCaps read GetDeviceCaps;

    function IndexOfDevice(ID: TGUID): Integer;

    property IsPlaying: Boolean read GetIsPlaying;
    property IsPaused: Boolean read GetIsPaused;

    property TrackLength: Integer read FLength;
    property TrackPosition: Integer read GetTrackPosition write SetTrackPosition;
  end;

implementation
uses Windows, Sysutils, ActiveX, ComObj;
{ TDSDMusic }

destructor TDSDMusic.Destroy;
var
  I: Integer;
begin
  StopTrack;
  if FDeviceList <> nil then
  begin
    for I:=0 to FDeviceList.Count - 1 do
      FreeMem(Pointer(FDeviceList.Objects[I]), SizeOf(TDMus_PortCaps));
    FDeviceList.Free; //Maybe Destroy?, nil is already checked for
  end;
  inherited Destroy;
end;

function TDSDMusic.GetDeviceList: TStringList;
var
  DMPerformance: IDirectMusicPerformance;
  DirectMusic  : IDirectMusic;
  Caps         : PDMus_PortCaps;
  I, Ret: Integer;
begin
  if FDeviceList = nil then
  begin  //Enumerate devices
    FDeviceList:=TStringList.Create;

    OleCheck(CoCreateInstance(CLSID_DirectMusicPerformance, nil, CLSCTX_INPROC,
                               IID_IDirectMusicPerformance, DMPerformance));
    OleCheck(DMPerformance.Init(DirectMusic, nil, 0));
    I:=0;
    repeat
      GetMem(Caps, SizeOf(TDMus_PortCaps));
      FillChar(Caps^, SizeOf(Caps),0);
      Caps^.dwSize:=SizeOf(TDMus_PortCaps);
      Ret:=DirectMusic.EnumPort(I, Caps^);
      OleCheck(Ret);
      if Ret = S_FALSE then
        Break
      else if Caps.dwClass = DMUS_PC_OUTPUTCLASS then   //We don't want input ports
        FDeviceList.AddObject(Caps^.wszDescription, TObject(Caps));
      Inc(I);
    until False;
  end;
  Result:=FDeviceList;
end;

function TDSDMusic.GetDeviceCaps(I: Integer): TDMus_PortCaps;
begin
  Result:=PDMus_PortCaps(DeviceList.Objects[I])^;
end;

procedure TDSDMusic.PlayTrack(Device: Integer; SGT, DLS: String; Effects: Cardinal);
var
  PortParams    : TDMus_PortParams;
  ObjectDesc    : TDMus_ObjectDesc;

  DirectMusic   : IDirectMusic;
  Port          : IDirectMusicPort;
  Loader        : IDirectMusicLoader;
  Collection    : IDirectMusicCollection;

  FilenameChars : array [0..DMUS_MAX_FILENAME-1] of Char;
begin
  StopTrack;

  ///////////////Load DirectMusic////////////////
  OleCheck(CoCreateInstance(CLSID_DirectMusicPerformance, nil, CLSCTX_INPROC,
                             IID_IDirectMusicPerformance, FPerformance));

  OleCheck(FPerformance.Init(DirectMusic, nil, 0));


  //////////////Create MIDI port/////////////////
  FillChar(PortParams,SizeOf(PortParams),0);
  with PortParams do
  begin
    dwSize:=SizeOf(PortParams);
    dwValidParams:=DMUS_PORTPARAMS_CHANNELGROUPS + DMUS_PORTPARAMS_EFFECTS;
    dwChannelGroups:=1;
    dwEffectFlags:=Effects;
  end;
  OleCheck(DirectMusic.CreatePort(DeviceCaps[Device].guidPort, PortParams, Port, nil));
  OleCheck(Port.Activate(True));
  OleCheck(DirectMusic.Activate(True));
  OleCheck(FPerformance.AddPort(Port));
  OleCheck(FPerformance.AssignPChannelBlock(0, Port, PortParams.dwChannelGroups));

  //////////////Loader object///////////////////

  OleCheck(CoCreateInstance(CLSID_DirectMusicLoader, nil, CLSCTX_INPROC,
   IID_IDirectMusicLoader, Loader));

  /////////////Load SGT/////////////////
  FillChar(ObjectDesc,SizeOf(ObjectDesc),0);
  with ObjectDesc do
  begin
    dwSize:=SizeOf(ObjectDesc);
    guidClass:=CLSID_DirectMusicSegment;
    StrPCopy(FilenameChars, SGT);
    MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, @FilenameChars, -1, wszFilename, MAX_PATH);
    dwValidData := DMUS_OBJ_CLASS or DMUS_OBJ_FULLPATH or DMUS_OBJ_FILENAME;
  end;
  OleCheck(Loader.GetObject(ObjectDesc, IID_IDirectMusicSegment, FSegment));


  OleCheck(FSegment.SetParam(GUID_Unload, $FFFFFFFF, 0, 0, Pointer(FPerformance)));

  ////////////////Load DLS if needed/////////////////
  if (DeviceCaps[Device].dwFlags and DMUS_PC_DLS) = DMUS_PC_DLS then
  begin
    FillChar(ObjectDesc,SizeOf(ObjectDesc), 0);
    with ObjectDesc do
    begin
      dwSize:=SizeOf(ObjectDesc);
      guidClass:=CLSID_DirectMusicCollection;
      StrPCopy(FilenameChars, DLS);
      MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, FilenameChars, -1, wszFilename, MAX_PATH );
      dwValidData := DMUS_OBJ_CLASS or DMUS_OBJ_FULLPATH or DMUS_OBJ_FILENAME;
    end;
    OleCheck(Loader.GetObject(ObjectDesc, IID_IDirectMusicCollection, Collection));
    OleCheck(FSegment.SetParam(GUID_ConnectToDLSCollection, $FFFFFFFF, 0, 0, Pointer(Collection)));
    OleCheck(FSegment.SetParam(GUID_Download, $FFFFFFFF, $0, 0, Pointer(FPerformance)));
  end;

  ///////////Play midi file/////////////////
  FPausedPos:=-1;
  OleCheck(FPerformance.PlaySegment(FSegment, DMUS_SEGF_BEAT, 0, @FSegmentState));
  OleCheck(FSegment.GetLength(FLength));
end;

procedure TDSDMusic.StopTrack;
begin
  if IsPlaying then
    FPerformance.Stop(nil, nil, 0, 0);
  FSegmentState:=nil;
  FSegment:=nil;
  FPerformance:=nil;
end;

procedure TDSDMusic.PauseTrack;
begin
  if IsPlaying and not IsPaused then
  begin
    FSegmentState.GetSeek(FPausedPos);
    FPerformance.Stop(nil, nil, 0, 0);
  end;
end;

procedure TDSDMusic.ResumeTrack;
begin
  if IsPlaying and IsPaused then
  begin
    FSegment.SetStartPoint(FPausedPos);
    OleCheck(FPerformance.PlaySegment(FSegment, DMUS_SEGF_BEAT, 0, @FSegmentState));
    FPausedPos:=-1;
  end;
end;

function TDSDMusic.GetTrackPosition: Integer;
begin
  if IsPlaying and not IsPaused then
    OleCheck(FSegmentState.GetSeek(Result))
  else
    Result:=FPausedPos;
end;

procedure TDSDMusic.SetTrackPosition(const Value: Integer);
begin
  if IsPlaying then
    if not IsPaused then
    begin
      PauseTrack;
      FPausedPos:=Value;
      ResumeTrack;
    end
    else
      FPausedPos:=Value;
end;

function TDSDMusic.GetIsPlaying: Boolean;
begin
  Result:=(FPerformance <> nil) and (FSegment <> nil) and (FSegmentState <> nil);
end;

function TDSDMusic.GetIsPaused: Boolean;
begin
  Result:=FPausedPos >= 0;
end;

function TDSDMusic.IndexOfDevice(ID: TGUID): Integer;
var
  I: Integer;
begin
  Result:=-1;
  for I:=0 to FDeviceList.Count - 1 do
    if IsEqualGUID(PDMus_PortCaps(FDeviceList.Objects[I])^.guidPort, ID) then
    begin
      Result:=I;
      Break;
    end;
end;

end.
