unit config;

{*************TFF8ConfigIni************
A class to write or read FF8\config.ini
Made in Delphi 4

Author:  Dag Sverre Seljebotn, dagsverre@sensewave.com
Created: 15-04-2000
History: 30-04-2000 First release

Created for the FF8 Configurator v. 2, http://ffsf.cjb.net

This code is distributed under the GNU General Public License.
NO WARRANTY. See COPYING for details.

Copyright (c) 2000 Dag Sverre Seljebotn
}
interface
  uses Windows, Sysutils, Registry, FF8Ini, ComObj, ActiveX, Classes;

type
  TGUIDSettings = (fgGraphics, fgSound, fgMusic);
  TBoolSettings = (fbHardware, fbQuarterScreen, fbHiResVideo, fbWorldColor,
                   fbWorldTexture, fbWorldFog, fbBilinear, fbHiResFonts,
                   fbReverseSpeakers, fbEAX, fbReverb, fbChorus, fbXG);

  TButtonSettings = (bsMenu, bsAction, bsMisc, bsCancel, bsToggle, bsTrigger,
                     bsRotLt, bsRotRt, bsStart, bsSelect, bsUp, bsDown,
                     bsLeft, bsRight);

  TStringSettings = (fsSoundfont, fsMusicSets);

  TFF8ConfigIni = class(TFF8IniFile)
  private
    FSgtOverrides: TStringList;

    procedure BackupSettings;
    procedure RestoreSettings;

    //Property getters/setters
    function GetBoolSettings(Setting: TBoolSettings): Boolean;
    procedure SetBoolSettings(Setting: TBoolSettings;
      const Value: Boolean);
    function GetGUIDSettings(Setting: TGUIDSettings): TGUID;
    procedure SetGUIDSettings(Setting: TGUIDSettings; const Value: TGUID);
    function GetJoystickSettings(Button: TButtonSettings): Integer;
    function GetKeyboardSettings(Button: TButtonSettings): Integer;
    procedure SetJoystickSettings(Button: TButtonSettings; const Value: Integer);
    procedure SetKeyboardSettings(Button: TButtonSettings; const Value: Integer);
    function GetStringSettings(Setting: TStringSettings): String;
    procedure SetStringSettings(Setting: TStringSettings;
      const Value: String);
    function GetAddonPath: String;
    procedure SetAddonPath(const Value: String);
  public
    destructor Destroy; override;

    procedure ActivateSettings;
    procedure DeactivateSettings;

    procedure BackupCLSIDs(CLSIDs: array of TGUID);
    procedure RestoreCLSIDs;


    property BoolSettings[Setting: TBoolSettings]: Boolean read GetBoolSettings
      write SetBoolSettings;
    property GUIDSettings[Setting: TGUIDSettings]: TGUID read GetGUIDSettings
      write SetGUIDSettings;
    property JoystickSettings[Button: TButtonSettings]: Integer read GetJoystickSettings
      write SetJoystickSettings;
    property KeyboardSettings[Button: TButtonSettings]: Integer read GetKeyboardSettings
      write SetKeyboardSettings;
    property StringSettings[Setting: TStringSettings]: String read GetStringSettings
      write SetStringSettings;

    property AddonPath: String read GetAddonPath write SetAddonPath;
  end;

const
  CLSIDSection    = 'DirectX';
  BackupSection   = 'Backup';
  SettingsSection = 'Settings';
  JoystickSection = 'Joystick';
  KeyboardSection = 'Keyboard';
  PathSection     = 'Paths';

  FF8RegKey = '\Software\Square Soft, Inc\FINAL FANTASY VIII\1.00\';

  FF8RegValuesInt: array[0..2] of String =
  ('Graphics','MidiOptions','SoundOptions');

  FF8RegValuesGUID: array[0..2] of String =
  ('GraphicsGUID','MIDIGUID','SoundGUID');

  FF8IniKeysBool: array[TBoolSettings] of String =
  ('Hardware_Renderer', 'Quarter_Screen', 'Hi-res_Video',
   'World_Color_Animation', 'World_Texture_Animation', 'World_Fog',
   'Bilinear_Filtering', 'Hi-res_Fonts', 'Reversed_Speakers', 'EAX', 'Reverb',
   'Chorus', 'Yamaha_XG');

  FF8IniKeysGUID: array[TGUIDSettings] of String =
  ('Graphics_Device', 'Sound_Device', 'Music_Device');

  FF8IniKeysString: array[TStringSettings] of String =
  ('Soundfont', 'MusicSets');

  FF8IniKeysControl: array[TButtonSettings] of String =
  ('Menu', 'Action', 'Misc', 'Exit', 'Toggle', 'Trigger', 'RotLt', 'RotRt',
   'Start', 'Select', 'Up', 'Down', 'Left', 'Right');

  FF8ControlsCFG: array[TButtonSettings] of String =
  ('1. "Select"   ', '2. "Exit"     ', '3. "Misc"     ', '4. "Menu"     ',
   '5. "Toggle"   ', '6. "Trigger"  ', '7. "RotLt"    ', '8. "RotRt"    ',
   '9. "Start"    ', '10. "Select"   ', '11. "Up"       ', '12. "Down"     ',
   '13. "Left"     ', '14. "Right"    ');


  ControlsCFG = 'ff8input.cfg';
  ControlsBAK = 'ff8input.bak';

  ValAppPath      = 'AppPath';
  ValDataDrive    = 'DataDrive';



implementation

{ TFF8ConfigIni }

procedure TFF8ConfigIni.ActivateSettings;
const
  //The GRAPHICS values in the registry...thanks to Jari Huttunen for these
  grHiResMovie   = $00000001;
  grNoMapFog     = $00000002;
  grNoMapColor   = $00000004;
  grNoMapTexture = $00000008;
  grNoBilinear   = $00000010;
  grHiResFont    = $00000020;
  grFullscreen   = $00100000;
  grHardware     = $10000000;

  //SoundOptions...
  sndEAX         = $00000004;
  sndReverse     = $00100000;

  //MidiOptions...
  midReverb      = $00000001;
  midChorus      = $00000002;
  midXG          = $00000010;


var
  GUID: TGUID;
  Flags: Cardinal;
  I: TButtonSettings;
begin
  BackupSettings;
  with TRegistry.Create do
  try
    RootKey:=HKEY_LOCAL_MACHINE;
    if not OpenKey(FF8RegKey, False) then
      raise Exception.Create('Final Fantasy not installed.');

    GUID:=GUIDSettings[fgGraphics];
    WriteBinaryData('GraphicsGUID', GUID, SizeOf(GUID));
    GUID:=GUIDSettings[fgSound];
    WriteBinaryData('SoundGUID', GUID, SizeOf(GUID));
    GUID:=GUIDSettings[fgMusic];
    WriteBinaryData('MIDIGUID', GUID, SizeOf(GUID));

    Flags:=0;
    if BoolSettings[fbHardware] then Flags:=Flags or grHardware;
    if not BoolSettings[fbQuarterScreen] then Flags:=Flags or grFullscreen;
    if BoolSettings[fbHiResVideo] then Flags:=Flags or grHiResMovie;
    if not BoolSettings[fbWorldColor] then Flags:=Flags or grNoMapColor;
    if not BoolSettings[fbWorldTexture] then Flags:=Flags or grNoMapTexture;
    if not BoolSettings[fbWorldFog] then Flags:=Flags or grNoMapFog;
    if not BoolSettings[fbBilinear] then Flags:=Flags or grNoBilinear;
    if BoolSettings[fbHiResFonts] then Flags:=Flags or grHiResFont;
    WriteInteger('Graphics', Flags);

    Flags:=0;
    if BoolSettings[fbReverseSpeakers] then Flags:=Flags or sndReverse;
    if BoolSettings[fbEAX] then Flags:=Flags or sndEAX;
    WriteInteger('SoundOptions', Flags);

    Flags:=0;
    if BoolSettings[fbReverb] then Flags:=Flags or midReverb;
    if BoolSettings[fbChorus] then Flags:=Flags or midChorus;
    if BoolSettings[fbXG] then Flags:=Flags or midXG;
    WriteInteger('MidiOptions', Flags);

    with TStringList.Create do
    try
      Add('Keyboard');
      for I:=Low(TButtonSettings) to High(TButtonSettings) do
        Add(FF8ControlsCFG[I] + IntToStr(KeyboardSettings[I]));
      Add('Joystick');
      for I:=Low(TButtonSettings) to High(TButtonSettings) do
        Add(FF8ControlsCFG[I] + IntToStr(JoystickSettings[I]));
      SaveToFile(Path + ControlsCFG);
    finally
      Free;
    end;
  finally
    Free;
  end;
end;

procedure TFF8ConfigIni.DeactivateSettings;
begin
  RestoreSettings;
end;

procedure TFF8ConfigIni.BackupCLSIDs(CLSIDs: array of TGUID);
var
  I: Integer;
  Reg: TRegistry;
  S: String;
begin
  Reg:=TRegistry.Create;
  try
    Reg.RootKey:=HKEY_CLASSES_ROOT;
    for I:=Low(CLSIDs) to High(CLSIDs) do
    begin
      S:=GUIDToString(CLSIDs[I]);
      if Reg.OpenKey('\CLSID\' + S + '\InprocServer32', False) then
        WriteString(CLSIDSection, S, Reg.ReadString(''));
    end;
  finally
    Reg.Free;
  end;
end;

procedure TFF8ConfigIni.RestoreCLSIDs;
var
  I: Integer;
  Section: TStringList;
  Reg: TRegistry;
  S: String;
begin
  Reg:=nil; Section:=nil;
  try
    Section:=TStringList.Create;
    Reg:=TRegistry.Create;
    Reg.RootKey:=HKEY_CLASSES_ROOT;

    ReadSection(CLSIDSection, Section);
    for I:=0 to Section.Count - 1 do
    begin
      S:=ReadString(CLSIDSection, Section[I], '');
      if Reg.OpenKey('\CLSID\' + Section[I] + '\InprocServer32', False) then
        Reg.WriteString('', S);
    end;
  finally
    Section.Free;
    Reg.Free;
  end;
end;

procedure TFF8ConfigIni.BackupSettings;
var
  I: Integer;
  GUID: TGUID;
  Reg: TRegistry;
begin
  CopyFile(PChar(Path + ControlsCFG), PChar(Path + ControlsBak), True);
  Reg:=TRegistry.Create;
  try
    Reg.RootKey:=HKEY_LOCAL_MACHINE;
    if Reg.OpenKey(FF8RegKey, False) then
    begin
      for I:=Low(FF8RegValuesInt) to High(FF8RegValuesInt) do
        if Reg.ValueExists(FF8RegValuesInt[I]) then
          WriteInteger(BackupSection, FF8RegValuesInt[I],
           Reg.ReadInteger(FF8RegValuesInt[I]));

      for I:=Low(FF8RegValuesGUID) to High(FF8RegValuesGUID) do
        if Reg.ValueExists(FF8RegValuesGUID[I]) then
        begin
          Reg.ReadBinaryData(FF8RegValuesGUID[I], GUID, SizeOf(GUID));
          Self.WriteString(BackupSection, FF8RegValuesGUID[I], GUIDToString(GUID));
        end;
    end;
  finally
    Reg.Free;
  end;
end;

procedure TFF8ConfigIni.RestoreSettings;
var
  I: Integer;
  Int: Integer;
  GUID: TGUID;
  Reg: TRegistry;
begin
  Reg:=TRegistry.Create;
  try
    Reg.RootKey:=HKEY_LOCAL_MACHINE;
    if Reg.OpenKey(FF8RegKey, False) then
    begin
      for I:=Low(FF8RegValuesInt) to High(FF8RegValuesInt) do
      begin
        Int:=ReadInteger(BackupSection, FF8RegValuesInt[I], Integer($FFFFFFFF)); //F is not a graphics option, so this should be safe
        if Int <> Integer($FFFFFFFF) then
          Reg.WriteInteger(FF8RegValuesInt[I], Int);
      end;

      for I:=Low(FF8RegValuesGUID) to High(FF8RegValuesGUID) do
      begin
        GUID:=StringToGUID(ReadString(BackupSection, FF8RegValuesGUID[I],
         '{00000000-0000-0000-C000-000000000046}')); //IUnknown isn't a device...(and {000...000] is used)
        if not IsEqualGUID(GUID, IUnknown) then
          Reg.WriteBinaryData(FF8RegValuesGUID[I], GUID, SizeOf(GUID));
      end;
    end;
  finally
    Reg.Free;
  end;
end;

function TFF8ConfigIni.GetBoolSettings(Setting: TBoolSettings): Boolean;
begin
  Result:=ReadBool(SettingsSection, FF8IniKeysBool[Setting], False)
end;

procedure TFF8ConfigIni.SetBoolSettings(Setting: TBoolSettings;
  const Value: Boolean);
begin
  WriteBool(SettingsSection, FF8IniKeysBool[Setting], Value);
end;


function TFF8ConfigIni.GetGUIDSettings(Setting: TGUIDSettings): TGUID;
begin
  Result:=StringToGUID(ReadString(SettingsSection, FF8IniKeysGUID[Setting],
    '{00000000-0000-0000-0000-000000000000}'));
end;

procedure TFF8ConfigIni.SetGUIDSettings(Setting: TGUIDSettings;
  const Value: TGUID);
begin
  WriteString(SettingsSection, FF8IniKeysGUID[Setting], GUIDToString(Value));
end;

function TFF8ConfigIni.GetStringSettings(Setting: TStringSettings): String;
begin
  Result:=ReadString(SettingsSection, FF8IniKeysString[Setting], '');
end;

procedure TFF8ConfigIni.SetStringSettings(Setting: TStringSettings;
  const Value: String);
begin
  WriteString(SettingsSection, FF8IniKeysString[Setting], Value);
end;

function TFF8ConfigIni.GetJoystickSettings(Button: TButtonSettings): Integer;
begin
  Result:=ReadInteger(JoystickSection, FF8IniKeysControl[Button], 0);
end;

function TFF8ConfigIni.GetKeyboardSettings(Button: TButtonSettings): Integer;
begin
  Result:=ReadInteger(KeyboardSection, FF8IniKeysControl[Button], 0);
end;

procedure TFF8ConfigIni.SetJoystickSettings(Button: TButtonSettings;
  const Value: Integer);
begin
  WriteInteger(JoystickSection, FF8IniKeysControl[Button], Value);
end;

procedure TFF8ConfigIni.SetKeyboardSettings(Button: TButtonSettings;
  const Value: Integer);
begin
  WriteInteger(KeyboardSection, FF8IniKeysControl[Button], Value);
end;

destructor TFF8ConfigIni.Destroy;
begin
  FSGTOverrides.Free;
  inherited Destroy;
end;

const
  AddonPacks = 'AddonPacks';

function TFF8ConfigIni.GetAddonPath: String;
begin
  Result:=ReadString(PathSection, AddonPacks, '');
  if (Result <> '') and (Result[Length(Result)] <> '\') then
    Result:=Result + '\';
end;

procedure TFF8ConfigIni.SetAddonPath(const Value: String);
begin
  WriteString(PathSection, AddonPacks, Value);
end;

end.
